/*
 * linescanCamera.c
 *
 *  Created on: 27 Jan 2025
 *      Author: user
 */

#include "linescanCamera.h"
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "MK66F18.h"
#include "fsl_debug_console.h"

// Global variables definition
uint16_t ImageData[IMAGE_SIZE];
volatile uint16_t currentPixelNumber = 0;
int ImageDataDifference[IMAGE_SIZE];
int CompareValue;
int i;
int BlackLineLeft;
int BlackLineRight;
int RoadMiddle;
int Deviation = 0;

void ADC1_IRQHANDLER(void) {
    uint32_t result_values[2] = {0};
    for (int i = 0; i < 2; i++) {
        uint32_t status = ADC16_GetChannelStatusFlags(ADC1_PERIPHERAL, i);
        if (status == kADC16_ChannelConversionDoneFlag) {
            result_values[i] = ADC16_GetChannelConversionValue(ADC1_PERIPHERAL, i);
        }
    }
    ImageData[currentPixelNumber] = result_values[0];
    currentPixelNumber++;
    #if defined _CORTEX_M && (_CORTEX_M == 4U)
        __DSB();
    #endif
}

void FTM1_IRQHANDLER(void) {
    uint32_t intStatus;
    static int16_t parallax1_si_count = 0;
    intStatus = FTM_GetStatusFlags(FTM1_PERIPHERAL);
    FTM_ClearStatusFlags(FTM1_PERIPHERAL, intStatus);
    parallax1_si_count++;
    if(parallax1_si_count == 1) {
        GPIO_PortClear(BOARD_PARALLAX1_S1_PTA9_GPIO, BOARD_PARALLAX1_S1_PTA9_GPIO_PIN_MASK);
        currentPixelNumber = 0;
    }
    if((parallax1_si_count >= 1) && (parallax1_si_count <= 128)) {
        ADC16_SetChannelConfig(ADC1_PERIPHERAL, ADC1_CH0_CONTROL_GROUP, &ADC1_channelsConfig[0]);
    }
    if(parallax1_si_count > SI_CYCLE_LIMIT) {
        parallax1_si_count = 0;
        GPIO_PortSet(BOARD_PARALLAX1_S1_PTA9_GPIO, BOARD_PARALLAX1_S1_PTA9_GPIO_PIN_MASK);
    }
    #if defined _CORTEX_M && (_CORTEX_M == 4U)
        __DSB();
    #endif
}

void calculateDeviation() {
    findBlackLineRight();
    findBlackLineLeft();
    RoadMiddle = (BlackLineRight + BlackLineLeft) / 2;
    Deviation = RoadMiddle - (IMAGE_SIZE / 2);
}

void findBlackLineRight() {
    for (i = 64; i < IMAGE_SIZE - 1; i++) {
        ImageDataDifference[i] = ImageData[i] - ImageData[i + 1];
    }
    CompareValue = THRESHOLD;
    for (i = 64; i < IMAGE_SIZE - 1; i++) {
        if (ImageDataDifference[i] > CompareValue) {
            CompareValue = ImageDataDifference[i];
            BlackLineRight = i;
        }
    }
}

void findBlackLineLeft() {
    for (i = 65; i > 0; i--) {
        ImageDataDifference[i] = ImageData[i] - ImageData[i - 1];
    }
    CompareValue = THRESHOLD;
    for (i = 65; i > 0; i--) {
        if (ImageDataDifference[i] > CompareValue) {
            CompareValue = ImageDataDifference[i];
            BlackLineLeft = i;
        }
    }
}

